/*
 * Quoridor server and related libraries
 * Copyright (C) 2009-2010 Jaroslav Tulach <jaroslav.tulach@apidesign.org>
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program. Look for COPYING file in the top folder.
 * If not, see http://www.gnu.org/licenses/.
 */

package cz.xelfi.quoridor.webidor.resources;

import com.sun.jersey.api.json.JSONWithPadding;
import java.io.File;
import java.io.FileInputStream;
import java.io.IOException;
import java.util.HashMap;
import java.util.Map;
import java.util.Properties;
import java.util.UUID;
import javax.annotation.ManagedBean;
import javax.inject.Singleton;
import javax.ws.rs.GET;
import javax.ws.rs.PUT;
import javax.ws.rs.Path;
import javax.ws.rs.Produces;
import javax.ws.rs.QueryParam;
import javax.ws.rs.WebApplicationException;
import javax.ws.rs.core.MediaType;
import javax.ws.rs.core.Response.Status;

/**
 *
 * @author Jaroslav Tulach <jtulach@netbeans.org>
 */
@Path("/")
@Singleton
@ManagedBean
public class Quoridor {
    private final File path;
    private Games games;
    private Users users;
    private final Map<UUID,String> loggedIn;

    public Quoridor() {
        String prop = System.getProperty("quoridor.dir"); // NOI18N
        if (prop == null) {
            prop = System.getProperty("user.dir") + File.separatorChar + ".quoridor"; // NOI18N
        }
        path = new File(prop);
        path.mkdirs();
        loggedIn = new HashMap<UUID, String>();
        games = new Games(new File(path, "games"), this); // NOI18N
        users = new Users(new File(path, "users"), this); // NOI18N
    }

    @Path("games")
    public Games getGames() {
        return games;
    }

    @Path("users")
    public Users getUsers() {
        return users;
    }

    @Path("login")
    @PUT
    @Produces({ MediaType.TEXT_PLAIN })
    public String login(
        @QueryParam("name") String name,
        @QueryParam("password") String password
    ) throws IOException {
        File f = new File(path, "passwd"); // NOI18Nt
        Properties p = new Properties();
        try {
            p.load(new FileInputStream(f));
        } catch (IOException ex) {
            ex.printStackTrace();
        }
        boolean loggedInOK = false;
        if (name != null && password.equals(p.getProperty(name))) {
            loggedInOK = true;
        } else {
            loggedInOK = getUsers().verifyPassword(name, password);
        }

        if (loggedInOK) {
            UUID uuid = UUID.randomUUID();
            loggedIn.put(uuid, name);
            return uuid.toString();
        } else {
            return null;
        }
        
    }

    @Path("login")
    @GET
    @Produces({ MediaType.TEXT_PLAIN })
    public String isLoggedIn(
        @QueryParam("id") String id
    ) {
        String ret = null;
        try {
            if (id != null) {
                ret = loggedIn.get(UUID.fromString(id));
            }
        } catch (IllegalArgumentException ex) {
            // OK, happens for invalid ids
        }
        return ret == null ? "" : ret;
    }

    @Path("login")
    @GET
    @Produces({ "application/x-javascript", MediaType.APPLICATION_JSON, MediaType.TEXT_XML })
    public JSONWithPadding loggedInInfo(
        @QueryParam("callback") String callback,
        @QueryParam("id") String id
    ) throws IOException {
        String ret = null;
        try {
            if (id != null) {
                ret = loggedIn.get(UUID.fromString(id));
            }
        } catch (IllegalArgumentException ex) {
            // OK, happens for invalid ids
        }
        if (ret == null) {
            throw new WebApplicationException(Status.UNAUTHORIZED);
        } else {
            return getUsers().getUserInfo(callback, id, ret);
        }
    }
}
