/*
 * Quoridor server and related libraries
 * Copyright (C) 2009-2010 Jaroslav Tulach <jaroslav.tulach@apidesign.org>
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program. Look for COPYING file in the top folder.
 * If not, see http://www.gnu.org/licenses/.
 */

package cz.xelfi.quoridor.webidor;

import com.sun.jersey.test.framework.WebAppDescriptor;
import com.sun.jersey.test.framework.AppDescriptor;
import com.sun.jersey.api.client.GenericType;
import java.util.List;
import com.sun.jersey.test.framework.JerseyTest;
import java.io.File;
import java.io.FileOutputStream;
import java.io.IOException;
import java.util.Properties;
import javax.ws.rs.core.MediaType;
import org.junit.Test;
import static org.junit.Assert.*;

/**
 *
 * @author Jaroslav Tulach <jtulach@netbeans.org>
 */
public class AllGamesTest extends JerseyTest {
    static {
        System.setProperty("JERSEY_HTTP_PORT", "45420");
    }

    private File dir;

    @Override
    protected AppDescriptor configure() {
        try {
            dir = File.createTempFile("quoridor", ".dir");
            dir.delete();
            System.setProperty("quoridor.dir", dir.getPath());
            dir.mkdirs();
            File passwd = new File(dir, "passwd");
            FileOutputStream os = new FileOutputStream(passwd);
            os.write("Jarda=heslo\nJirka=pesko\nJan=pan\n".getBytes("UTF-8"));
            os.close();
        } catch (Exception ex) {
            throw new IllegalStateException(ex);
        }
        return new WebAppDescriptor.Builder("cz.xelfi.quoridor.webidor.resources").contextPath("allgamess").build();
    }

    @Override
    public void tearDown() throws Exception {
        deleteRec(dir);
    }

    static void deleteRec(File dir) throws IOException {
        if (dir == null) {
            return;
        }
        File[] arr = dir.listFiles();
        if (arr != null) {
            for (File f : arr) {
                deleteRec(f);
            }
        }
        dir.delete();
    }
    @Test public void testListGames() throws Exception {
        File usersDir = new File(dir, "users");
        usersDir.mkdirs();
        File fJarda = new File(usersDir, "Jarda");
        {
            Properties p = new Properties();
            p.setProperty("email", "jar@da.cz");
            p.setProperty("permission.games", "true");
            p.store(new FileOutputStream(fJarda), "");
        }
        File fJirka = new File(usersDir, "Jirka");
        {
            Properties p = new Properties();
            p.setProperty("email", "jir@ka.cz");
            p.store(new FileOutputStream(fJirka), "");
        }
        File fJan = new File(usersDir, "Jan");
        {
            Properties p = new Properties();
            p.setProperty("email", "j@an.cz");
            p.store(new FileOutputStream(fJan), "");
        }

        String logRoot = resource().path("login").
            queryParam("name", "Jarda").
            queryParam("password", "heslo").
            accept(MediaType.TEXT_PLAIN).
            put(String.class);
        String logJirka = resource().path("login").
            queryParam("name", "Jirka").
            queryParam("password", "pesko").
            accept(MediaType.TEXT_PLAIN).
            put(String.class);
        String logJan = resource().path("login").
            queryParam("name", "Jan").
            queryParam("password", "pan").
            accept(MediaType.TEXT_PLAIN).
            put(String.class);

        GameId s = resource().path("games").queryParam("white", "Jan")
                .queryParam("loginID", logJan)
                .queryParam("black", "Jirka").post(GameId.class);
            resource().path("games/" + s.getId())
                .queryParam("loginID", logJan)
                .queryParam("player", "Jan").queryParam("move", "RESIGN").put(GameId.class);

        GenericType<List<GameId>> gType = new GenericType<List<GameId>>() {};
        List<GameId> all = resource().path("games/").queryParam("loginID", logRoot).accept(MediaType.TEXT_XML).get(gType);
        boolean found = false;
        for (GameId id : all) {
            if (id.getId().equals(s.getId())) {
                found = true;
                break;
            }
        }
        assertTrue("List of games shall contai all games: " + all, found);

        Game end = resource().path("games/" + s.getId()).queryParam("loginID", logRoot).accept(MediaType.TEXT_XML).get(Game.class);
        assertEquals("One can see status of games with priviledges", GameStatus.blackWon, end.getId().getStatus());
    }
}
