/*
 * Quoridor server and related libraries
 * Copyright (C) 2009-2010 Jaroslav Tulach <jaroslav.tulach@apidesign.org>
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program. Look for COPYING file in the top folder.
 * If not, see http://www.gnu.org/licenses/.
 */
package cz.xelfi.quoridor.webidor;

import com.sun.jersey.test.framework.WebAppDescriptor;
import com.sun.jersey.test.framework.AppDescriptor;
import java.io.FileInputStream;
import com.sun.jersey.api.client.UniformInterfaceException;
import com.sun.jersey.test.framework.JerseyTest;
import java.io.File;
import java.io.FileOutputStream;
import java.io.IOException;
import java.util.Properties;
import javax.ws.rs.core.MediaType;
import org.junit.Test;
import static org.junit.Assert.*;

/**
 *
 * @author Jaroslav Tulach <jtulach@netbeans.org>
 */
public class UsersTest extends JerseyTest {
    static {
        System.setProperty("JERSEY_HTTP_PORT", "39434");
    }

    private File dir;

    @Override
    protected AppDescriptor configure() {
        try {
            dir = File.createTempFile("quoridor", ".dir");
            dir.delete();
            System.setProperty("quoridor.dir", dir.getPath());
            dir.mkdirs();
            File passwd = new File(dir, "passwd");
            FileOutputStream os = new FileOutputStream(passwd);
            os.write("Jarda=heslo\n".getBytes("UTF-8"));
            os.close();
            File usersDir = new File(dir, "users");
            usersDir.mkdirs();
            File fJirka = new File(usersDir, "Jirka");
            {
                Properties p = new Properties();
                p.setProperty("email", "jir@ka.cz");
                p.setProperty("passwd", "pesko");
                p.store(new FileOutputStream(fJirka), "");
            }
        } catch (Exception ex) {
            throw new IllegalStateException(ex);
        }
        return new WebAppDescriptor.Builder("cz.xelfi.quoridor.webidor.resources").contextPath("userstest").build();
    }

    @Override
    public void tearDown() throws Exception {
        deleteRec(dir);
    }

    static void deleteRec(File dir) throws IOException {
        if (dir == null) {
            return;
        }
        File[] arr = dir.listFiles();
        if (arr != null) {
            for (File f : arr) {
                deleteRec(f);
            }
        }
        dir.delete();
    }

    @Test public void testListUsers() throws Exception {
        File usersDir = new File(dir, "users");
        usersDir.mkdirs();
        File fJarda = new File(usersDir, "Jarda");
        {
            Properties p = new Properties();
            p.setProperty("email", "jar@da.cz");
            p.setProperty("permission.email", "true");
            p.store(new FileOutputStream(fJarda), "");
        }
        File fJirka = new File(usersDir, "Jirka");
        {
            Properties p = new Properties();
            p.load(new FileInputStream(fJirka));
            p.setProperty("email", "jir@ka.cz");
            p.store(new FileOutputStream(fJirka), "");
        }

        String logJarda = resource().path("login").
            queryParam("name", "Jarda").
            queryParam("password", "heslo").
            accept(MediaType.TEXT_PLAIN).
            put(String.class);
        String logJirka = resource().path("login").
            queryParam("name", "Jirka").
            queryParam("password", "pesko").
            accept(MediaType.TEXT_PLAIN).
            put(String.class);

        User uJirka = resource().path("users/Jirka").accept(MediaType.TEXT_XML).get(User.class);
        assertEquals("Jirka", uJirka.getId());
        assertNull("Cannot get email without login", uJirka.getProperty("email"));


        uJirka = resource().path("users/Jirka").queryParam("loginID", logJirka).accept(MediaType.TEXT_XML).get(User.class);
        assertEquals("Jirka", uJirka.getId());
        assertEquals("Email for ownself is OK", "jir@ka.cz", uJirka.getProperty("email"));

        uJirka = resource().path("users/Jirka").queryParam("loginID", logJarda).accept(MediaType.TEXT_XML).get(User.class);
        assertEquals("Jirka", uJirka.getId());
        assertEquals("Jarda has permission for property email", "jir@ka.cz", uJirka.getProperty("email"));

        String txt = resource().path("users/Jirka").queryParam("loginID", logJarda).accept(MediaType.TEXT_XML).get(String.class);
        if (!txt.contains("<user id=\"Jirka\"><property name=\"email\">jir@ka.cz</property></user>")) {
            fail(txt);
        }

        try {
            resource().path("users/Jarda").queryParam("loginID", logJirka).
                    queryParam("name", "email").queryParam("value", "ka@jir.cz").accept(MediaType.TEXT_XML).post();
            fail("You cannot change email without priviledges");
        } catch (UniformInterfaceException e) {
            // OK, not allowed
        }

        resource().path("users/Jirka").queryParam("loginID", logJirka).
                queryParam("name", "email").queryParam("value", "ka@jir.cz").accept(MediaType.TEXT_XML).post();

        uJirka = resource().path("users/Jirka").queryParam("loginID", logJirka).accept(MediaType.TEXT_XML).get(User.class);
        assertEquals("Jirka", uJirka.getId());
        assertEquals("Email for ownself is OK", "ka@jir.cz", uJirka.getProperty("email"));

        try {
            resource().path("users/Jirka").queryParam("loginID", logJirka).
                    queryParam("name", "permission.email").queryParam("value", "true").accept(MediaType.TEXT_XML).post();
            fail("Shall not be allowed to change own permissions");
        } catch (UniformInterfaceException ex) {
            // OK, not allowed
        }
    }
}
